/*
**  CDSound.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU Lesser General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "SGSystem.h"
#include "CDSound.h"
#include <Foundation/NSValue.h>

@implementation CDSound

- (NSNumber *) currentTime
{
  int seconds = [[SGSystem sharedSystem] currentTimeOfCDTrack];
  /* Detect whether CD stop playing */
  if ((startPlaying == NO) && (seconds > 0))
    startPlaying = YES;
  if (startPlaying && (seconds < 1)) {
    startPlaying = NO;
    [[SGSystem sharedSystem] closeCD];
    [_delegate sound: self didFinishPlaying: YES];
  }
  return [NSNumber numberWithInt: seconds];
}

- (NSNumber *) duration
{
  ASSIGN(duration, [NSNumber numberWithInt: [[SGSystem sharedSystem] lengthOfCDTrack: [self track]]]);
  return duration;
}

- (void) setDuration: (NSNumber *) seconds
{
  /* No need to set duration */
}

- (BOOL) isPlaying
{
  return [[SGSystem sharedSystem] isCDPlaying];
}
- (BOOL) play
{
  return [[SGSystem sharedSystem] playCD: [self track]];
}

- (BOOL) seek: (int) seconds
{
  NSLog(@"CDSound: seek %d", [self track]);
  return [[SGSystem sharedSystem] playCD: [self track] at: seconds];
}

- (BOOL) pause
{
  return [[SGSystem sharedSystem] pauseCD];
}

- (BOOL) stop
{
  BOOL ret = [[SGSystem sharedSystem] stopCD];

  NSLog(@"CD stop playing");

  if (startPlaying) {
    startPlaying = NO;
    [_delegate sound: self didFinishPlaying: YES];
  }
  return ret;
}

- (BOOL) resume
{
  return [[SGSystem sharedSystem] resumeCD];
}

- (void) setTrack: (int) track
{
  _track = track;
}

- (int) track
{
  return _track;
}

- (id) init
{
  self = [super init];
  _track = 0;
  startPlaying = NO;
  return self;
}

- (void) dealloc
{
  [super dealloc];
}

@end
