/*
**  OGGEditor.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU Lesser General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "OGGEditor.h"
#include <vorbis/vorbisfile.h>
#include "vcedit.h"

@implementation OGGEditor

- (void) writeToFile: (NSString *) name
{
  int i;
  FILE *out;
  vcedit_state *state;
  vorbis_comment *vc;
  NSString *newfile;

  RETAIN(name);

  fseek(in, 0L, SEEK_SET);

  state = vcedit_new_state();

  if (vcedit_open(state, in) < 0) {
//      NSLog(@"vcedit_open failed: %@", filename);
      vcedit_clear(state);
      return;
  }    

  vc = vcedit_comments(state);
  vorbis_comment_clear(vc);
  vorbis_comment_init(vc);

  vorbis_comment_add_tag(vc, "title", (char *)[[data objectForKey: @"Title"] UTF8String]);
  vorbis_comment_add_tag(vc, "artist", (char *)[[data objectForKey: @"Artist"] UTF8String]);
  vorbis_comment_add_tag(vc, "album", (char *)[[data objectForKey: @"Album"] UTF8String]);

/*
  for(i = 0; i < state->vc->comments; i++)
    {
      NSLog(@"New: %s", state->vc->user_comments[i]);
    }
*/

  if ([name isEqualToString: [data objectForKey: @"File Name"]])
    {
      newfile = [name stringByAppendingString:@"_temp"];
    }
  else
    {
      newfile = name;
    }

  out = fopen([newfile cString], "w");

  if (out == NULL) {
      NSLog(@"fopen failed: %@", newfile);
      fclose(out);
      return;
  }
  
  if(vcedit_write(state, out) < 0)
    {
      NSLog(@"Failed to write comments to file %@", newfile);
    }

  vorbis_comment_clear(vc);
  vcedit_clear(state);
  fclose(out);

  if ([name isEqualToString: [data objectForKey: @"File Name"]])
    {
      NSFileManager *manager = [NSFileManager defaultManager];
      [manager removeFileAtPath: name handler: nil];
      [manager movePath: newfile toPath: name handler: nil];
    }
  RELEASE(name);
}

- (void) dealloc
{
  ov_clear(&vf);
  fclose(in);
  RELEASE(data);
  [super dealloc];
}

- (id) initWithFile: (NSString *) name
{
  int i;
  vorbis_info *vi;
  vorbis_comment *vc;

  self = [super init];
  data = [NSMutableDictionary new];
  [data setObject: name forKey: @"File Name"];

  in = fopen([[data objectForKey: @"File Name"] cString], "r");

  if(ov_open(in, &vf, NULL, 0) < 0)
    {
      NSLog(@"ov_open Can't open %@", name);
      return nil;
    }

  [data setObject: [NSNumber numberWithInt: (int)ov_time_total(&vf, -1)]
           forKey: @"Seconds"];

  vc = ov_comment(&vf, -1);

  for (i = 0; i < vc->comments; i++)
    {
      NSString *comment = [NSString stringWithUTF8String: vc->user_comments[i]];
      array = [comment componentsSeparatedByString: @"="];
      count = (int)[array count];
//      NSLog(@"Count=%d", count);
      if ([array count] != 2)
        continue;
      comment = [[[array objectAtIndex: 0] lowercaseString] stringByTrimmingSpaces];

      if ([comment isEqualToString: @"title"]) {
        [data setObject: [[array objectAtIndex: 1] stringByTrimmingSpaces]
                 forKey: @"Title"];
        continue;
      }
      if ([comment isEqualToString: @"artist"]) {                             
        [data setObject: [[array objectAtIndex: 1] stringByTrimmingSpaces]
                 forKey: @"Artist"]; 
        continue;
      }
      if ([comment isEqualToString: @"album"]) {                             
        [data setObject: [[array objectAtIndex: 1] stringByTrimmingSpaces]
                 forKey: @"Album"];
        continue;
      }
      if ([comment isEqualToString: @"date"]) {                             
        [data setObject: [[array objectAtIndex: 1] stringByTrimmingSpaces]
                 forKey: @"Year"];
        continue;
      }
      if ([comment isEqualToString: @"tracknumber"]) {                             
        [data setObject: [[array objectAtIndex: 1] stringByTrimmingSpaces]
                 forKey: @"Track Number"];
        continue;
      }
      if ([comment isEqualToString: @"genre"]) {                             
        [data setObject: [[array objectAtIndex: 1] stringByTrimmingSpaces]
                 forKey: @"Genre"];
        continue;
      }
      if ([comment isEqualToString: @"comment"]) {                             
        [data setObject: [[array objectAtIndex: 1] stringByTrimmingSpaces]
                 forKey: @"Comment"];
        continue;
      }
    }

  if ([data objectForKey: @"Title"] == nil)
    [data setObject: [name lastPathComponent] forKey: @"Title"];
  if ([data objectForKey: @"Artist"] == nil)
    [data setObject: @"Unknown Artist" forKey: @"Artist"];
  if ([data objectForKey: @"Album"] == nil)
    [data setObject: @"Unknown Album" forKey: @"Album"];
  if ([data objectForKey: @"Year"] == nil)
    [data setObject: @"Year Unknown" forKey: @"Year"];
  if ([data objectForKey: @"Track Number"] == nil)
    [data setObject: @"-" forKey: @"Track Number"];
  if ([data objectForKey: @"Genre"] == nil)
    [data setObject: @"Unknown Genre" forKey: @"Genre"];
  if ([data objectForKey: @"Comment"] == nil)
    [data setObject: @"-" forKey: @"Comment"];

  [data setObject: [NSString stringWithUTF8String: vc->vendor] forKey: @"Vendor"];

  vi = ov_info(&vf, -1);
  [data setObject: [NSNumber numberWithInt: vi->channels] forKey: @"Channels"];
  [data setObject: [NSNumber numberWithFloat: vi->rate/1000.0] forKey: @"Rate"];

  if (vi->bitrate_nominal > 0)
    [data setObject: [NSNumber numberWithDouble: vi->bitrate_nominal/1000.0]
             forKey: @"Bitrate Nominal"];

  if (vi->bitrate_upper > 0)
    [data setObject: [NSNumber numberWithDouble: vi->bitrate_upper/1000.0]
             forKey: @"Bitrate Upper"];

  if (vi->bitrate_lower > 0)
    [data setObject: [NSNumber numberWithDouble: vi->bitrate_lower/1000.0]
             forKey: @"Bitrate Lower"];

  [data setObject: [NSNumber numberWithLong: (long)ov_pcm_total(&vf, -1)]
           forKey: @"Decoded Length"];

  if ([data objectForKey: @"Bitrate Upper"] == nil)
    [data setObject: @"Not Set" forKey: @"Bitrate Upper"];

  if ([data objectForKey: @"Bitrate Lower"] == nil)
    [data setObject: @"Not Set" forKey: @"Bitrate Lower"];

  vorbis_comment_clear(vc);
  vorbis_info_clear(vi);

  return self;
}

- (NSNumber *) decodedLength
{
  return [data objectForKey: @"Decoded Length"];
}

- (NSNumber *) lowerOfBitrate
{
  return [data objectForKey: @"Bitrate Lower"];
}

- (NSNumber *) upperOfBitrate
{
  return [data objectForKey: @"Bitrate Upper"];
}

- (NSNumber *) nominalOfBitrate
{
  return [data objectForKey: @"Bitrate Nominal"];
}

- (NSNumber *) rate
{
  return [data objectForKey: @"Rate"];
}

- (NSNumber *) channels
{
  return [data objectForKey: @"Channels"];
}

- (NSString *) vendor
{
  return [data objectForKey: @"Vendor"];
}

- (NSString *) title
{
  return [data objectForKey: @"Title"];
}

- (NSString *) artist
{
  return [data objectForKey: @"Artist"];
}

- (NSString *) year
{
  return [data objectForKey: @"Year"];
}

- (NSString *) trackNumber
{
  return [data objectForKey: @"Track Number"];
}

- (NSString *) genre
{
  return [data objectForKey: @"Genre"];
}

- (NSString *) comment
{
  return [data objectForKey: @"Comment"];
}

- (void) setTitle: (NSString *) string
{
  [data setObject: string forKey: @"Title"];
}


- (void) setArtist: (NSString *) string
{
  [data setObject: string forKey: @"Artist"];
}

- (NSString *) album
{
  return [data objectForKey: @"Album"];
}

- (void) setAlbum: (NSString *) string
{
  [data setObject: string forKey: @"Album"];
}

- (NSNumber *) seconds
{
  return [data objectForKey: @"Seconds"];
}

- (NSString *) filename
{
  return [data objectForKey: @"File Name"];
}

- (NSString *) description
{
  return [NSString stringWithFormat: @"%@-%@-%@-%@-%@", 
                         [data objectForKey: @"File Name"],
                         [data objectForKey: @"Title"],
                         [data objectForKey: @"Artist"],
                         [data objectForKey: @"Album"],
                         [data objectForKey: @"Seconds"]];
}

@end
