/*
**  SGSound.m
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju Chen <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU Lesser General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "SGSystem.h"
#include "SGSound.h"
#include <Foundation/NSValue.h>
#include <Foundation/NSArray.h>

@implementation SGSound

- (BOOL) seek: (int) seconds
{
  [[SGSystem sharedSystem] seekSound: seconds];
  return YES;
}

- (BOOL) play
{
  NSLog(@"SGSound play %@", path);
  [[SGSystem sharedSystem] setDelegate: self];
  return [[SGSystem sharedSystem] playSound: path];
}

- (BOOL) stop
{
  return [[SGSystem sharedSystem] stopSound];
}

- (BOOL) pause
{
  return [[SGSystem sharedSystem] pauseSound];
}

- (BOOL) isPlaying
{
  return [[SGSystem sharedSystem] isSoundPlaying];
}

- (BOOL) isPaused
{
  return [[SGSystem sharedSystem] isSoundPaused];
}

- (BOOL) resume
{
  return [[SGSystem sharedSystem] resumeSound];
}

- (id) initWithContentsOfFile: (NSString *) filename byReference: (BOOL) aBool
{
  self = [super init];

  path = [filename stringByExpandingTildeInPath];
  RETAIN(path);
  duration = nil;

  return self;
}

- (NSNumber *) currentTime
{
  int seconds = [[SGSystem sharedSystem] currentTimeOfSound];
  return [NSNumber numberWithInt: seconds];
}

- (NSNumber *) duration
{
  if (duration == nil)
    {
      int seconds = [[SGSystem sharedSystem] lengthOfSound: path];
      [self setDuration: [NSNumber numberWithInt: seconds]];
    }
  return duration;
}

- (void) setDuration: (NSNumber *) seconds
{
  ASSIGN(duration, seconds);
}

- (void) dealloc
{
  RELEASE(duration);
  RELEASE(path);
  [super dealloc];
}

- (void) soundFinishPlaying
{
  NSLog(@"%@ Finish Playing", self);
  if (_delegate) {
    [_delegate sound: self didFinishPlaying: YES];
  }
}

/* Block other NSSound interface */
- (id) initWithContentsOfURL:(NSURL *)url byReference:(BOOL)byRef
{
  [self notImplemented: _cmd];
  return nil;
}

- (id)initWithData:(NSData *)data
{
  [self notImplemented: _cmd];
  return nil;
}

- (id)initWithPasteboard:(NSPasteboard *)pasteboard
{
  [self notImplemented: _cmd];
  return nil;
}

+ (BOOL)canInitWithPasteboard:(NSPasteboard *)pasteboard
{
  return NO;
}

+ (NSArray *)soundUnfilteredPasteboardTypes
{
  return [NSArray arrayWithObjects: @"NSGeneralPboardType", nil];
}

- (void)writeToPasteboard:(NSPasteboard *)pasteboard
{
  [self notImplemented: _cmd];
}

+ (NSArray *)soundUnfilteredFileTypes
{
  return [[SGSystem sharedSystem] availableFormats];
}

- (void)encodeWithCoder:(NSCoder *)coder
{
  [self notImplemented: _cmd];
}

- (id)initWithCoder:(NSCoder*)decoder
{
  [self notImplemented: _cmd];
  return nil;
}

- (id)copyWithZone:(NSZone *)zone
{
  [self notImplemented: _cmd];
  return nil;
}

@end

