/*
**  SongInfo.h
**
**  Copyright (c) 2002, 2003
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "SongInfo.h"
#include "SGSound.h"
#include <Foundation/Foundation.h>
#include <AppKit/NSSound.h>

@implementation SongInfo

- (id) copyWithZone: (NSZone *) zone
{
  SongInfo *newInfo = [[self class] allocWithZone: zone];
  [newInfo init];

  [newInfo setSong: [self song]];
  [newInfo setStatus: [self status]];
  [newInfo setArtist: [self artist]];
  [newInfo setAlbum: [self album]];
  [newInfo setTrack: [self track]];
  [newInfo setChecked: [self checked]];
  [newInfo setSound: [self sound]];
  [newInfo setFilename: [self filename]];
  [newInfo setDuration: [self duration]];

  return AUTORELEASE(newInfo);
}

- (void) encodeWithCoder: (NSCoder *)coder
{
  [coder encodeObject: info];
// Checked box
  [coder encodeValueOfObjCType: "s" at: &checked];
}

- (id) initWithCoder: (NSCoder *) coder
{
  self = [self init];

  [info setDictionary: [coder decodeObject]];
// Checked box
  [coder decodeValueOfObjCType: "s" at: &checked];

  return self;
}

- (void) updateDuration
{
  int time;
  if ([self sound] == nil)
    [self loadSound];
  time = [[(SGSound *)[self sound] duration] intValue]; 

  // If media is Audio CD, duration will be nil
  if (time) {
    [self setDuration: [NSNumber numberWithInt: time]];
  }
}

- (void) loadSound
{
  [self setSound: [[SGSound alloc] initWithContentsOfFile: [self filename]
                                              byReference: NO]];
}

- (id) init
{
  self = [super init];

  info = [NSMutableDictionary new];
  [info setObject: @"" forKey: @"status"];
  [info setObject: @"" forKey: @"song"];
  [info setObject: @"Unknown Artist" forKey: @"artist"];
  [info setObject: @"Unknown Album" forKey: @"album"];
  [info setObject: @"" forKey: @"filename"];
  [info setObject: [NSNumber numberWithInt: 0] forKey: @"track"];
  [info setObject: [NSNumber numberWithInt: 0] forKey: @"duration"];
  [self setChecked: NO];

  sound = nil;

  return self;
}

- (void) dealloc
{
  RELEASE(info);
  RELEASE(sound);
  [super dealloc];
}

- (NSString *) description
{
  return [NSString stringWithFormat: @"%@-%@-%@",
                                     [self song],
                                     [self artist],
                                     [self album]];
}

- (void) setSong: (NSString *) song
{
  if (song)
    [info setObject: song forKey: @"song"];
}

- (NSString *) song
{
  return [info objectForKey: @"song"];
}

- (void) setStatus: (id) status
{
  if (status)
    [info setObject: status forKey: @"status"];
}

- (id) status
{
  return [info objectForKey: @"status"];
}

- (void) setArtist: (NSString *) artist
{
  if (artist)
    [info setObject: artist forKey: @"artist"];
}

- (NSString *) artist
{
  return [info objectForKey: @"artist"];
}

- (void) setAlbum: (NSString *) album
{
  if (album)
    [info setObject: album forKey: @"album"];
}

- (NSString *) album
{
  return [info objectForKey: @"album"];
}

- (void) setTrack: (NSNumber *) track
{
  if (track)
    [info setObject: track forKey: @"track"];
}

- (NSNumber *) track
{
  return [info objectForKey: @"track"];
}

- (void) setChecked: (BOOL) bool_
{
  checked = bool_;
}

- (BOOL) checked
{
  return checked;
}

- (void) setSound: (NSSound *) snd
{
  if (snd)
    {
      ASSIGN(sound, snd);
    }
}

- (NSSound *) sound
{
  return sound;
}

- (void) setFilename: (NSString *) filename
{
  if (filename)
    [info setObject: filename forKey: @"filename"];
}

- (NSString *) filename
{
  return [info objectForKey: @"filename"];
}

- (void) setDuration: (NSNumber *) duration
{
  if (duration)
    [info setObject: duration forKey: @"duration"];
}

- (NSNumber *) duration
{
  return [info objectForKey: @"duration"];
}

- (NSNumber *) size
{
  NSFileManager *manager = [NSFileManager defaultManager];
  NSDictionary *attributes = [manager fileAttributesAtPath: [self filename]
                                  traverseLink: YES];
  return [attributes objectForKey: NSFileSize];
}

@end
